@@ -15,9 +15,12 @@ DOCUMENTATION = """
     author:
       - CyberArk BizDev (@cyberark-bizdev)
     description:
-      Retrieves credentials from Conjur using the controlling host's Conjur identity
-      or environment variables.
-      Environment variables could be CONJUR_ACCOUNT, CONJUR_APPLIANCE_URL, CONJUR_CERT_FILE, CONJUR_AUTHN_LOGIN, CONJUR_AUTHN_API_KEY, CONJUR_AUTHN_TOKEN_FILE
+      Retrieves credentials from Conjur using the controlling host's Conjur identity,
+      environment variables, or extra-vars.
+      Environment variables could be CONJUR_ACCOUNT, CONJUR_APPLIANCE_URL, CONJUR_CERT_FILE, CONJUR_CERT_CONTENT,
+      CONJUR_AUTHN_LOGIN, CONJUR_AUTHN_API_KEY, CONJUR_AUTHN_TOKEN_FILE
+      Extra-vars could be conjur_account, conjur_appliance_url, conjur_cert_file, conjur_cert_content,
+      conjur_authn_login, conjur_authn_api_key, conjur_authn_token_file
       Conjur info - U(https://www.conjur.org/).
     requirements:
       - 'The controlling host running Ansible has a Conjur identity.
@@ -46,16 +49,6 @@ DOCUMENTATION = """
             key: identity_file_path
         env:
           - name: CONJUR_IDENTITY_FILE
-      authn_token_file:
-        description: Path to the access token file.
-        type: path
-        default: /var/run/conjur/access-token
-        required: False
-        ini:
-          - section: conjur,
-            key: authn_token_file
-        env:
-          - name: CONJUR_AUTHN_TOKEN_FILE
       config_file:
         description: Path to the Conjur configuration file. The configuration file is a YAML file.
         type: path
@@ -66,6 +59,83 @@ DOCUMENTATION = """
             key: config_file_path
         env:
           - name: CONJUR_CONFIG_FILE
+      conjur_appliance_url:
+        description: Conjur appliance url
+        type: string
+        required: False
+        ini:
+          - section: conjur,
+            key: appliance_url
+        vars:
+          - name: conjur_appliance_url
+        env:
+          - name: CONJUR_APPLIANCE_URL
+      conjur_authn_login:
+        description: Conjur authn login
+        type: string
+        required: False
+        ini:
+          - section: conjur,
+            key: authn_login
+        vars:
+          - name: conjur_authn_login
+        env:
+          - name: CONJUR_AUTHN_LOGIN
+      conjur_account:
+        description: Conjur account
+        type: string
+        required: False
+        ini:
+          - section: conjur,
+            key: account
+        vars:
+          - name: conjur_account
+        env:
+          - name: CONJUR_ACCOUNT
+      conjur_authn_api_key:
+        description: Conjur authn api key
+        type: string
+        required: False
+        ini:
+          - section: conjur,
+            key: authn_api_key
+        vars:
+          - name: conjur_authn_api_key
+        env:
+          - name: CONJUR_AUTHN_API_KEY
+      conjur_cert_file:
+        description: Path to the Conjur cert file
+        type: path
+        required: False
+        ini:
+          - section: conjur,
+            key: cert_file
+        vars:
+          - name: conjur_cert_file
+        env:
+          - name: CONJUR_CERT_FILE
+      conjur_cert_content:
+        description: Content of the Conjur cert
+        type: string
+        required: False
+        ini:
+          - section: conjur,
+            key: cert_content
+        vars:
+          - name: conjur_cert_content
+        env:
+          - name: CONJUR_CERT_CONTENT
+      conjur_authn_token_file:
+        description: Path to the access token file
+        type: path
+        required: False
+        ini:
+          - section: conjur,
+            key: authn_token_file
+        vars:
+          - name: conjur_authn_token_file
+        env:
+          - name: CONJUR_AUTHN_TOKEN_FILE
 """
 
 EXAMPLES = """
@@ -85,25 +155,100 @@ RETURN = """
       - Value stored in Conjur.
 """
 
-import os.path
+import os
 import socket
+import yaml
+import traceback
+import ssl
+import re
+import ansible.module_utils.six.moves.urllib.error as urllib_error
 from ansible.errors import AnsibleError
 from ansible.plugins.lookup import LookupBase
 from base64 import b64encode
 from netrc import netrc
-from os import environ
-from time import time, sleep
+from time import sleep
 from ansible.module_utils.six.moves.urllib.parse import quote
-from ansible.module_utils.urls import urllib_error
 from stat import S_IRUSR, S_IWUSR
 from tempfile import gettempdir, NamedTemporaryFile
-import yaml
+try:
+    from cryptography.x509 import load_pem_x509_certificate
+    from cryptography.hazmat.backends import default_backend
+except ImportError:
+    CRYPTOGRAPHY_IMPORT_ERROR = traceback.format_exc()
+else:
+    CRYPTOGRAPHY_IMPORT_ERROR = None
+
 
 from ansible.module_utils.urls import open_url
 from ansible.utils.display import Display
-import ssl
 
 display = Display()
+temp_cert_file = None
+
+
+def _validate_pem_certificate(cert_content):
+    # Normalize line endings
+    if '\r\n' in cert_content:
+        cert_content = cert_content.replace('\r\n', '\n').strip()
+    elif '\r' in cert_content:
+        cert_content = cert_content.replace('\r', '\n').strip()
+    cert_content = re.sub(r'^[ \t]+', '', cert_content, flags=re.M)
+    cert_content = re.sub(r'[ \t]+$', '', cert_content, flags=re.M)
+    cert_content = re.sub(r'\n+', '\n', cert_content)
+
+    if not re.match(r"^-----BEGIN CERTIFICATE-----.+-----END CERTIFICATE-----$", cert_content, re.DOTALL):
+        raise AnsibleError("Invalid Certificate format.")
+
+    try:
+        load_pem_x509_certificate(cert_content.encode(), default_backend())
+        return cert_content
+    except ValueError as e:
+        raise AnsibleError(f"Invalid certificate content provided: {str(e)}. Please check the certificate format.")
+    except ssl.SSLError as e:
+        raise AnsibleError(f"SSL error while validating the certificate: {str(e)}. The certificate may be corrupted or invalid.")
+    except Exception as e:
+        raise AnsibleError(f"An error occurred while validating the certificate: {str(e)}. Please verify the certificate format and try again.")
+
+
+def _get_valid_certificate(cert_content, cert_file):
+    if cert_content:
+        try:
+            display.vvv("Validating provided certificate content")
+            cert_content = _validate_pem_certificate(cert_content)
+            return cert_content
+        except AnsibleError as e:
+            display.warning(f"Invalid certificate content: {str(e)}. Attempting to use certificate file.")
+
+    # If cert_content is invalid or missing, fall back to cert_file
+    if cert_file:
+        if not os.path.exists(cert_file):
+            raise AnsibleError(f"Certificate file `{cert_file}` does not exist or cannot be found.")
+        try:
+            with open(cert_file, 'rb') as f:
+                cert_file_content = f.read().decode('utf-8')
+                cert_file_content = _validate_pem_certificate(cert_file_content)
+                return cert_file_content
+        except Exception as e:
+            raise AnsibleError(f"Failed to load or validate certificate file `{cert_file}`: {str(e)}")
+
+    # If both cert_content and cert_file are missing or invalid, raise an error
+    raise AnsibleError("Both certificate content and certificate file are invalid or missing. Please provide a valid certificate.")
+
+
+def _get_certificate_file(cert_content, cert_file):
+    global temp_cert_file
+    cert_content = _get_valid_certificate(cert_content, cert_file)
+
+    if cert_content:
+        try:
+            temp_cert_file = NamedTemporaryFile(delete=False, mode='w', encoding='utf-8')
+            temp_cert_file.write(cert_content)
+            temp_cert_file.close()
+            cert_file = temp_cert_file.name
+        except Exception as e:
+            raise AnsibleError(f"Failed to create temporary certificate file: {str(e)}")
+
+    return cert_file
 
 
 # Load configuration and return as dictionary if file is present on file system
@@ -273,7 +418,26 @@ class LookupModule(LookupBase):
         elif not terms[0] or terms[0].isspace():
             raise AnsibleError("Invalid secret path: empty secret path not accepted.")
 
-        self.set_options(direct=kwargs)
+        # We should register the variables as LookupModule options.
+        #
+        # Doing this has some nice advantages if we're considering supporting
+        # a set of Ansible variables that could sometimes replace environment
+        # variables.
+        #
+        # Registering the variables as options forces them to adhere to the
+        # behavior described in the DOCUMENTATION variable. An option can have
+        # both a Ansible variable and environment variable source, which means
+        # Ansible will do some juggling on our behalf.
+        self.set_options(var_options=variables, direct=kwargs)
+
+        appliance_url = self.get_var_value("conjur_appliance_url")
+        account = self.get_var_value("conjur_account")
+        authn_login = self.get_var_value("conjur_authn_login")
+        authn_api_key = self.get_var_value("conjur_authn_api_key")
+        cert_file = self.get_var_value("conjur_cert_file")
+        cert_content = self.get_var_value("conjur_cert_content")
+        authn_token_file = self.get_var_value("conjur_authn_token_file")
+
         validate_certs = self.get_option('validate_certs')
         conf_file = self.get_option('config_file')
         as_file = self.get_option('as_file')
@@ -281,53 +445,62 @@ class LookupModule(LookupBase):
         if validate_certs is False:
             display.warning('Certificate validation has been disabled. Please enable with validate_certs option.')
 
-        if 'http://' in str(environ.get("CONJUR_APPLIANCE_URL")):
+        if 'http://' in str(appliance_url):
             raise AnsibleError(('[WARNING]: Conjur URL uses insecure connection. Please consider using HTTPS.'))
 
+        if validate_certs is True:
+            cert_file = _get_certificate_file(cert_content, cert_file)
+
         conf = _merge_dictionaries(
             _load_conf_from_file(conf_file),
             {
-                "account": environ.get('CONJUR_ACCOUNT'),
-                "appliance_url": environ.get("CONJUR_APPLIANCE_URL")
+                "account": account,
+                "appliance_url": appliance_url
             } if (
-                environ.get('CONJUR_ACCOUNT') is not None
-                and environ.get('CONJUR_APPLIANCE_URL') is not None
+                account is not None
+                and appliance_url is not None
             )
             else {},
             {
-                "cert_file": environ.get('CONJUR_CERT_FILE')
-            } if (environ.get('CONJUR_CERT_FILE') is not None)
+                "cert_file": cert_file
+            } if (cert_file is not None)
             else {},
             {
-                "authn_token_file": environ.get('CONJUR_AUTHN_TOKEN_FILE')
-            } if (environ.get('CONJUR_AUTHN_TOKEN_FILE') is not None)
+                "authn_token_file": authn_token_file
+            } if authn_token_file is not None
             else {}
         )
 
+        if 'account' not in conf or 'appliance_url' not in conf:
+            raise AnsibleError(
+                """Configuration must define options `conjur_account` and `conjur_appliance_url`.
+                This config can be set by any of the following methods, listed in order of priority:
+                - Ansible variables of the same name, set either in the parent playbook or passed to
+                  the ansible-playbook command with the --extra-vars flag
+                - Environment variables `CONJUR_ACCOUNT` and `CONJUR_APPLIANCE_URL`
+                - A configuration file on the controlling host with fields `account` and `appliance_url`"""
+            )
+
         if 'authn_token_file' not in conf:
             identity_file = self.get_option('identity_file')
             identity = _merge_dictionaries(
                 _load_identity_from_file(identity_file, conf['appliance_url']),
                 {
-                    "id": environ.get('CONJUR_AUTHN_LOGIN'),
-                    "api_key": environ.get('CONJUR_AUTHN_API_KEY')
-                } if (environ.get('CONJUR_AUTHN_LOGIN') is not None
-                      and environ.get('CONJUR_AUTHN_API_KEY') is not None)
+                    "id": authn_login,
+                    "api_key": authn_api_key
+                } if authn_login is not None
+                and authn_api_key is not None
                 else {}
             )
 
-            if 'account' not in conf or 'appliance_url' not in conf:
-                raise AnsibleError(
-                    ("Configuration file on the controlling host must "
-                     "define `account` and `appliance_url`"
-                     "entries or they should be environment variables")
-                )
-
             if 'id' not in identity or 'api_key' not in identity:
                 raise AnsibleError(
-                    ("Identity file on the controlling host must contain "
-                     "`login` and `password` entries for Conjur appliance"
-                     " URL or they should be environment variables")
+                    """Configuration must define options `conjur_authn_login` and `conjur_authn_api_key`.
+                    This config can be set by any of the following methods, listed in order of priority:
+                    - Ansible variables of the same name, set either in the parent playbook or passed to
+                      the ansible-playbook command with the --extra-vars flag
+                    - Environment variables `CONJUR_AUTHN_LOGIN` and `CONJUR_AUTHN_API_KEY`
+                    - An identity file on the controlling host with the fields `login` and `password`"""
                 )
 
         cert_file = None
@@ -335,33 +508,49 @@ class LookupModule(LookupBase):
             display.vvv("Using cert file path {0}".format(conf['cert_file']))
             cert_file = conf['cert_file']
 
-        token = None
-        if 'authn_token_file' not in conf:
-            token = _fetch_conjur_token(
+        try:
+            token = None
+            if 'authn_token_file' not in conf:
+                token = _fetch_conjur_token(
+                    conf['appliance_url'],
+                    conf['account'],
+                    identity['id'],
+                    identity['api_key'],
+                    validate_certs,
+                    cert_file
+                )
+            else:
+                if not os.path.exists(conf['authn_token_file']):
+                    raise AnsibleError('Conjur authn token file `{0}` was not found on the host'
+                                       .format(conf['authn_token_file']))
+                with open(conf['authn_token_file'], 'rb') as f:
+                    token = f.read()
+
+            conjur_variable = _fetch_conjur_variable(
+                terms[0],
+                token,
                 conf['appliance_url'],
                 conf['account'],
-                identity['id'],
-                identity['api_key'],
                 validate_certs,
                 cert_file
             )
-        else:
-            if not os.path.exists(conf['authn_token_file']):
-                raise AnsibleError('Conjur authn token file `{0}` was not found on the host'
-                                   .format(conf['authn_token_file']))
-            with open(conf['authn_token_file'], 'rb') as f:
-                token = f.read()
-
-        conjur_variable = _fetch_conjur_variable(
-            terms[0],
-            token,
-            conf['appliance_url'],
-            conf['account'],
-            validate_certs,
-            cert_file
-        )
+        finally:
+            if temp_cert_file:
+                try:
+                    if os.path.exists(temp_cert_file.name):
+                        os.unlink(temp_cert_file.name)
+                except (OSError, PermissionError) as e:
+                    raise AnsibleError(f"Failed to delete temporary certificate file `{temp_cert_file.name}`: {str(e)}")
 
         if as_file:
             return _store_secret_in_file(conjur_variable)
 
         return conjur_variable
+
+    def get_var_value(self, key):
+        try:
+            variable_value = self.get_option(key)
+        except KeyError:
+            raise AnsibleError("{0} was not defined in configuration".format(key))
+
+        return variable_value
